# Copyright 2021 SUSE LLC
#
# This file is part of gceimgutils
#
# gceimgutils is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# gceimgutils is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with gcelistimg. If not, see <http://www.gnu.org/licenses/>.

import logging
import json

import gceimgutils.gceutils as utils

from gceimgutils.gceimgutils import GCEImageUtils
from gceimgutils.gceimgutilsExceptions import GCEListImgException


class GCEListImage(GCEImageUtils):
    """Class to list images from GCE project"""

    # ---------------------------------------------------------------------
    def __init__(
            self,
            credentials_path=None,
            credentials_info=None,
            image_name=None,
            image_name_fragment=None,
            image_name_match=None,
            log_callback=None,
            log_level=logging.INFO,
            project=None,
            detail=False
    ):
        GCEImageUtils.__init__(
            self,
            project,
            credentials_path,
            credentials_info,
            log_level,
            log_callback
        )

        self.image_name = image_name
        self.image_name_fragment = image_name_fragment
        self.image_name_match = image_name_match
        self.detail = detail

    # ---------------------------------------------------------------------
    def get_images(self):
        """Get images from sdk"""
        owned_images = utils.get_project_images(
            self.images_client,
            self.project,
            True
        )

        if self.image_name:
            return utils.find_images_by_name(
                owned_images,
                self.image_name,
                self.log
            )
        elif self.image_name_fragment:
            return utils.find_images_by_name_fragment(
                owned_images,
                self.image_name_fragment,
                self.log
            )
        elif self.image_name_match:
            try:
                return utils.find_images_by_name_regex_match(
                    owned_images,
                    self.image_name_match,
                    self.log
                )
            except Exception:
                msg = 'Unable to complie regular expression "%s"'
                msg = msg % self.image_name_match
                raise GCEListImgException(msg)
        else:
            return owned_images

    # ---------------------------------------------------------------------
    def list_images(self):
        """List the images"""
        images = self.get_images()

        if not images:
            raise GCEListImgException('No Images found')

        for image in images:
            if self.detail or self.image_name:
                self.log.info(json.dumps(utils.image_to_dict(image), indent=4))
            else:
                self.log.info(image.name)



import pytest

from unittest.mock import patch, MagicMock

from gceimgutils.gcedeprecateimg import GCEDeprecateImage


class TestGCEDeprecateImage(object):
    def setup_method(self, method):
        self.images_client = MagicMock()
        self.logger = MagicMock()
        self.operation = MagicMock()
        self.images_client.deprecate.return_value = self.operation

        self.kwargs = {
            'image_name': 'image123',
            'project': 'fakeproject',
            'credentials_path': 'test/data/creds.json',
            'log_callback': self.logger
        }

        self.deprecater = GCEDeprecateImage(**self.kwargs)

    @patch('gceimgutils.gceutils.compute_v1')
    @patch('gceimgutils.gceutils.AuthorizedSession')
    def test_deprecate_image(self, mock_auth_session, mock_compute_v1):
        mock_compute_v1.ImagesClient.return_value = self.images_client

        self.operation.error_code = None
        self.operation.warnings = None
        self.operation.result.return_value = 0

        self.deprecater.deprecate_image()

    @patch('gceimgutils.gceutils.compute_v1')
    @patch('gceimgutils.gceutils.AuthorizedSession')
    def test_deprecate_exc(self, mock_auth_session, mock_compute_v1):
        mock_compute_v1.ImagesClient.return_value = self.images_client

        self.operation.error_code = None
        self.operation.warnings = None
        self.operation.result.return_value = 0
        self.images_client.deprecate.side_effect = Exception(
            'Invalid credentials!'
        )

        with pytest.raises(Exception) as error:
            self.deprecater.deprecate_image()

        msg = 'Unable to deprecate image: "image123". Invalid credentials!'
        assert str(error.value) == msg



# Copyright (c) 2021 SUSE LLC
#
# This file is part of gceimgutils.
#
# gceimgutils is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# gceimgutils is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with gceimgutils.ase.  If not, see <http://www.gnu.org/licenses/>.

import logging

import gceimgutils.gceutils as utils


class GCEImageUtils():
    """Base class for GCE Image Utilities"""

    # ---------------------------------------------------------------------
    def __init__(
        self,
        project,
        credentials_path=None,
        credentials_info=None,
        log_level=logging.INFO,
        log_callback=None
    ):

        self.project = project
        self.credentials_path = credentials_path
        self.credentials_info = credentials_info
        self._credentials = None
        self._images_client = None
        self._regions_client = None
        self._zones_client = None
        self._storage_client = None

        if log_callback:
            self.log = log_callback
        else:
            logger = logging.getLogger('gceimgutils')
            logger.setLevel(log_level)
            self.log = logger

        try:
            self.log_level = self.log.level
        except AttributeError:
            self.log_level = self.log.logger.level  # LoggerAdapter

    # ---------------------------------------------------------------------
    @property
    def images_client(self):
        """Get an authenticated images client"""
        if not self._images_client:
            self._images_client = utils.get_images_client(
                self.credentials
            )

        return self._images_client

    # ---------------------------------------------------------------------
    @property
    def regions_client(self):
        """Get an authenticated regions client"""
        if not self._regions_client:
            self._regions_client = utils.get_regions_client(
                self.credentials
            )

        return self._regions_client

    # ---------------------------------------------------------------------
    @property
    def zones_client(self):
        """Get an authenticated zones client"""
        if not self._regions_client:
            self._zones_client = utils.get_zones_client(
                self.credentials
            )

        return self._zones_client

    # ---------------------------------------------------------------------
    @property
    def storage_client(self):
        """Get an authenticated storage client"""
        if not self._storage_client:
            self._storage_client = utils.get_storage_client(
                self.project,
                self.credentials
            )

        return self._storage_client

    # ---------------------------------------------------------------------
    @property
    def credentials(self):
        if not self._credentials:
            self._credentials = utils.get_credentials(
                self.project,
                self.credentials_path,
                self.credentials_info
            )

        return self._credentials



import pytest

from unittest.mock import patch, MagicMock

from gceimgutils.gcecreateimg import GCECreateImage


class TestGCECreateImage(object):
    def setup_method(self, method):
        self.images_client = MagicMock()
        self.logger = MagicMock()
        self.operation = MagicMock()

        image = MagicMock()
        image.status = 'READY'

        self.images_client.insert.return_value = self.operation
        self.images_client.get.return_value = image

        self.kwargs = {
            'image_name': 'image123',
            'project': 'fakeproject',
            'image_description': 'Test image',
            'family': 'images',
            'bucket_name': 'my-bucket',
            'object_name': 'image.tar.gz',
            'guest_os_features': [
                'UEFI_COMPATIBLE',
                'VIRTIO_SCSI_MULTIQUEUE',
                'GVNIC'
            ],
            'licenses': [
                'projects/project-123/global/licenses/'
                'cloud-marketplace-1234567890123456-1234567890123456'
            ],
            'credentials_path': 'test/data/creds.json',
            'log_callback': self.logger
        }

        self.creator = GCECreateImage(**self.kwargs)

    @patch('gceimgutils.gceutils.time')
    @patch('gceimgutils.gceutils.compute_v1')
    @patch('gceimgutils.gceutils.AuthorizedSession')
    def test_create_image(
        self,
        mock_auth_session,
        mock_compute_v1,
        mock_time
    ):
        mock_compute_v1.ImagesClient.return_value = self.images_client

        self.operation.error_code = None
        self.operation.warnings = None
        self.operation.result.return_value = 0

        self.creator.create_image()

    @patch('gceimgutils.gceutils.time')
    @patch('gceimgutils.gceutils.compute_v1')
    @patch('gceimgutils.gceutils.AuthorizedSession')
    def test_create_exc(
        self,
        mock_auth_session,
        mock_compute_v1,
        mock_time
    ):
        mock_compute_v1.ImagesClient.return_value = self.images_client

        self.operation.error_code = None
        self.operation.warnings = None
        self.operation.result.return_value = 0
        self.images_client.insert.side_effect = Exception(
            'Invalid credentials!'
        )

        with pytest.raises(Exception) as error:
            self.creator.create_image()

        msg = 'Unable to create image: "image123". Invalid credentials!'
        assert str(error.value) == msg



# Copyright 2025 SUSE LLC
#
# This file is part of gceimgutils
#
# gceimgutils is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# gceimgutils is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with gcecreateimg. If not, see <http://www.gnu.org/licenses/>.

import logging

from gceimgutils.gceutils import (
    wait_on_image_ready,
    wait_on_operation
)

from gceimgutils.gceimgutils import GCEImageUtils
from gceimgutils.gceimgutilsExceptions import GCECreateImgException

from google.cloud import compute_v1


class GCECreateImage(GCEImageUtils):
    """Class to create images from GCE project"""

    # ---------------------------------------------------------------------
    def __init__(
        self,
        image_name,
        image_description,
        bucket_name,
        object_name,
        architecture='x86_64',
        family=None,
        guest_os_features=None,
        licenses=None,
        credentials_path=None,
        credentials_info=None,
        project=None,
        log_callback=None,
        log_level=logging.INFO,
    ):
        GCEImageUtils.__init__(
            self,
            project,
            credentials_path,
            credentials_info,
            log_level,
            log_callback
        )

        self.image_name = image_name
        self.image_description = image_description
        self.family = family
        self.blob_uri = ''.join([
            'https://www.googleapis.com/storage/v1/b/',
            bucket_name,
            '/o/',
            object_name
        ])
        self.architecture = architecture
        self.guest_os_features = guest_os_features
        self.licenses = licenses

    # ---------------------------------------------------------------------
    def create_image(self):
        """Create the image"""
        mapping = {
            'name': self.image_name,
            'description': self.image_description,
            'raw_disk': {'source': self.blob_uri},
            'architecture': self.architecture.upper()
        }

        if self.guest_os_features:
            mapping['guest_os_features'] = [
                {'type_': feature} for feature in self.guest_os_features
            ]

        if self.family:
            mapping['family'] = self.family

        if self.licenses:
            mapping['licenses'] = self.licenses

        image = compute_v1.Image(mapping)

        try:
            operation = self.images_client.insert(
                project=self.project,
                image_resource=image
            )
        except Exception as error:
            msg = f'Unable to create image: "{self.image_name}". {str(error)}'
            self.log.error(msg)
            raise GCECreateImgException(msg) from error

        wait_on_operation(operation, self.log, 'image creation')
        return wait_on_image_ready(
            self.images_client,
            project=self.project,
            image_name=self.image_name
        )



#!/usr/bin/python3
#
# Copyright (c) 2021 SUSE LLC.  All rights reserved.
#
# This file is part of gceimgutils
#
# gceutils is free software: you can redistribute it
# and/or modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation, either version 3 of
# the License, or (at your option) any later version.
#
# gceutils is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with gceutils. If not, see
# <http://www.gnu.org/licenses/>.
#

import logging
import pytest
import os

from google.cloud import compute_v1
from unittest.mock import Mock

from gceimgutils import gceutils
from gceimgutils.gceimgutilsExceptions import (
    GCEProjectCredentialsException
)

this_path = os.path.dirname(os.path.abspath(__file__))
data_path = os.path.dirname(this_path) + os.sep + 'data'

logger = logging.getLogger('gceimgutils')
logger.setLevel(logging.INFO)


# --------------------------------------------------------------------
def test_find_images_by_name_find_one():
    images = _get_test_images()
    found_images = gceutils.find_images_by_name(
        images, 'sles15sp2hpc-test-v20210131', logger
    )
    assert 1 == len(found_images)
    assert found_images[0].name == 'sles15sp2hpc-test-v20210131'


# --------------------------------------------------------------------
def test_find_images_by_name_find_multiple():
    images = _get_test_images()
    found_images = gceutils.find_images_by_name(
        images, 'nested-vm-image', logger
    )
    assert 2 == len(found_images)
    assert found_images[0].name == 'nested-vm-image'
    assert found_images[1].name == 'nested-vm-image'


# --------------------------------------------------------------------
def test_find_images_by_name_find_none():
    images = _get_test_images()
    found_images = gceutils.find_images_by_name(
        images, 'foo', logger
    )
    assert 0 == len(found_images)


# --------------------------------------------------------------------
def test_find_images_by_name_fragment_find_one():
    images = _get_test_images()
    found_images = gceutils.find_images_by_name_fragment(
        images, 'hpc', logger
    )
    assert 1 == len(found_images)
    assert found_images[0].name == 'sles15sp2hpc-test-v20210131'


# --------------------------------------------------------------------
def test_find_images_by_name_fragment_find_multiple():
    images = _get_test_images()
    found_images = gceutils.find_images_by_name_fragment(
        images, 'opensuse', logger
    )
    # The image data we read also contains dleted images and the test helper
    # does not filter
    assert 21 == len(found_images)
    for image in found_images:
        assert 'opensuse' in image.name


# --------------------------------------------------------------------
def test_find_images_by_name_fragment_find_none():
    images = _get_test_images()
    found_images = gceutils.find_images_by_name_fragment(
        images, 'foo', logger
    )
    assert 0 == len(found_images)


# --------------------------------------------------------------------
def test_find_images_by_name_regex_find_one():
    images = _get_test_images()
    found_images = gceutils.find_images_by_name_regex_match(
        images, '.*hpc.*', logger
    )
    assert 1 == len(found_images)
    assert found_images[0].name == 'sles15sp2hpc-test-v20210131'


# --------------------------------------------------------------------
def test_find_images_by_name_regex_find_multiple():
    images = _get_test_images()
    found_images = gceutils.find_images_by_name_regex_match(
        images, 'sles', logger
    )
    assert 11 == len(found_images)


# --------------------------------------------------------------------
def test_find_images_by_name_regex_find_none():
    images = _get_test_images()
    found_images = gceutils.find_images_by_name_regex_match(
        images, 'foo', logger
    )
    assert 0 == len(found_images)


# --------------------------------------------------------------------
def test_get_credentials_no_args():
    try:
        gceutils.get_credentials()
    except GCEProjectCredentialsException as ex:
        expected_msg = 'Either project name, credentials file path or '
        expected_msg += 'credentials object must be given'
        assert expected_msg == format(ex)


# --------------------------------------------------------------------
def test_get_credentials_no_credfile():
    try:
        gceutils.get_credentials(credentials_file='foo')
    except GCEProjectCredentialsException as ex:
        expected_msg = 'Provided credentials file "foo" not found'
        assert expected_msg == format(ex)


# --------------------------------------------------------------------
def test_get_credentials_no_credfile_default_loc():
    test_def_config = os.path.expanduser('~/.config/gce/foo.json')
    try:
        gceutils.get_credentials('foo')
    except GCEProjectCredentialsException as ex:
        expected_msg = '"%s" credentials not found' % test_def_config
        assert expected_msg == format(ex)


# --------------------------------------------------------------------
def test_get_credentials_format_error():
    cred_file = data_path + '/fake_credentials.json'
    try:
        gceutils.get_credentials(
            credentials_file=cred_file
        )
    except GCEProjectCredentialsException as ex:
        expected_msg = 'Could not load credentials: ' \
                       'Service account info was not in the ' \
                       'expected format'
        assert expected_msg in format(ex)


# --------------------------------------------------------------------
# Helpers
def _get_test_images():
    """Read the stored image data and return as list"""

    data = eval(open(data_path + '/image_data.txt').read())
    images = []

    for image in data:
        images.append(compute_v1.Image(image))

    return images


# --------------------------------------------------------------------
def test_str_to_bool():
    result = gceutils.str_to_bool('y')
    assert result

    result = gceutils.str_to_bool('n')
    assert not result

    with pytest.raises(ValueError):
        gceutils.str_to_bool('invalid')


# --------------------------------------------------------------------
def test_get_region_list():
    region = Mock()
    region.status = 'UP'
    region.name = 'us-east1'
    region.zones = ['us-east1-a']
    regions = [region]
    regions_client = Mock()
    regions_client.list.return_value = regions

    result = gceutils.get_region_list(regions_client, 'test-project')
    assert result == {'us-east1-a'}


# --------------------------------------------------------------------
def test_get_zones():
    zone = Mock()
    zone.region = 'us-east1'
    zone.name = 'us-east1-a'
    zones = [zone]
    zones_client = Mock()
    zones_client.list.return_value = zones

    result = gceutils.get_zones(zones_client, 'test-project')
    assert result == ['zones/us-east1-a']


# --------------------------------------------------------------------
def test_blob_exists():
    blob = Mock()
    blob.exists.return_value = True
    bucket = Mock()
    bucket.blob.return_value = blob
    storage_client = Mock()
    storage_client.bucket.return_value = bucket

    result = gceutils.blob_exists(storage_client, 'bucket', 'blob')
    assert result



# Copyright 2021 SUSE LLC
#
# This file is part of gceimgutils
#
# gceimgutils is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# gceimgutils is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with gceremoveimg. If not, see <http://www.gnu.org/licenses/>.

import logging
import sys

import gceimgutils.gceutils as utils

from gceimgutils.gceimgutils import GCEImageUtils
from gceimgutils.gceimgutilsExceptions import GCERemoveImgException


class GCERemoveImage(GCEImageUtils):
    """Class to remove images from GCE project"""

    # ---------------------------------------------------------------------
    def __init__(
            self,
            confirm=False,
            credentials_path=None,
            credentials_info=None,
            image_name=None,
            image_name_fragment=None,
            image_name_match=None,
            log_callback=None,
            log_level=logging.INFO,
            project=None,
            remove_all=False
    ):
        GCEImageUtils.__init__(
            self,
            project,
            credentials_path,
            credentials_info,
            log_level,
            log_callback
        )

        self.confirm = confirm
        self.image_name = image_name
        self.image_name_fragment = image_name_fragment
        self.image_name_match = image_name_match
        self.remove_all = remove_all

    # ---------------------------------------------------------------------
    def _check_images_boundary_condition(self, images):
        """Check if the images found meet operating conditions:
           If all is true we delete all found images if not we should have
           found only one image"""

        if not images:
            self.log.info(
                'No images to remove found'
            )

        if len(images) > 1 and not self.remove_all:
            msg = 'Found multiple images to remove, but "all" is '
            msg += 'not set. Cannot disambiguate images to remove'
            self.log.info(msg)
            return False

        return True

    # ---------------------------------------------------------------------
    def _get_images_to_remove(self):
        """Find the images to remove"""
        owned_images = utils.get_project_images(
            self.images_client,
            self.project,
            True
        )
        if self.image_name:
            return utils.find_images_by_name(
                owned_images,
                self.image_name,
                self.log
            )
        elif self.image_name_fragment:
            return utils.find_images_by_name_fragment(
                owned_images,
                self.image_name_fragment,
                self.log
            )
        elif self.image_name_match:
            try:
                return utils.find_images_by_name_regex_match(
                    owned_images,
                    self.image_name_match,
                    self.log
                )
            except Exception:
                msg = 'Unable to complie regular expression "%s"'
                msg = msg % self.image_name_match
                raise GCERemoveImgException(msg)
        else:
            msg = 'No remove image condition set. Should not reach '
            msg += 'this point.'
            raise GCERemoveImgException(msg)

    # ---------------------------------------------------------------------
    def print_remove_info(self):
        """Print information about images that would be deleted"""
        images = self._get_images_to_remove()
        images_ok = self._check_images_boundary_condition(images)

        if not images_ok:
            # While the boundary conditions are not met, we are just printing
            # information thus this is not considered a failure
            return True

        header_msg = 'Would remove image'
        if len(images) > 1:
            header_msg += 's'

        self.log.info(header_msg)
        for image in images:
            self.log.info(f'\t\t{image.name}')

        return True

    # ---------------------------------------------------------------------
    def remove_images(self):
        """Remove the images"""
        images = self._get_images_to_remove()
        images_ok = self._check_images_boundary_condition(images)

        if not images_ok:
            raise GCERemoveImgException('Image ambiguity')

        for image in images:
            try:
                self.remove_image(image.name)
            except GCERemoveImgException:
                pass  # If an image fails it is logged

    # ---------------------------------------------------------------------
    def remove_image(self, image_name):
        """Remove the image"""
        delete = True
        if self.confirm:
            delete = self._query_yes_no(image_name)

        if delete:
            try:
                operation = self.images_client.delete(
                    project=self.project,
                    image=image_name
                )
            except Exception as error:
                msg = f'Unable to remove image: "{image_name}". {str(error)}'
                self.log.error(msg)
                raise GCERemoveImgException(msg) from error

            utils.wait_on_operation(operation, self.log, 'image deletion')

    # ---------------------------------------------------------------------
    def _query_yes_no(self, image):
        yes = {'yes', 'y', 'ye', ''}
        no = {'no', 'n'}

        while True:
            try:
                choice = input(
                    f'\tConfirm Delete Image: {image.name}\t (Y/n)'
                ).lower()
                if choice in yes:
                    return True
                elif choice in no:
                    return False
                else:
                    sys.stdout.write("Please respond with 'yes' or 'no'\n")
            except KeyboardInterrupt:
                raise GCERemoveImgException(
                    'Keyboard Interrupt received, exiting'
                )



# Copyright 2026 SUSE LLC
#
# This file is part of gceimgutils
#
# gceimgutils is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# gceimgutils is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with gcegetblob. If not, see <http://www.gnu.org/licenses/>.

import logging

from gceimgutils.gceimgutils import GCEImageUtils
from gceimgutils.gceimgutilsExceptions import GCEGetBlobException


class GCEGetBlob(GCEImageUtils):
    """Class to download blobs from a bucket in a GCE project"""

    # ---------------------------------------------------------------------
    def __init__(
        self,
        blob_name,
        bucket_name,
        destination_file_name,
        credentials_path=None,
        credentials_info=None,
        project=None,
        log_callback=None,
        log_level=logging.INFO,
    ):
        GCEImageUtils.__init__(
            self,
            project,
            credentials_path,
            credentials_info,
            log_level,
            log_callback
        )

        self.blob_name = blob_name
        self.bucket_name = bucket_name
        self.destination_file_name = destination_file_name

    # ---------------------------------------------------------------------
    def get_blob(self):
        """get the blob"""
        bucket = self.storage_client.bucket(self.bucket_name)

        try:
            blob = bucket.blob(
                blob_name=self.blob_name,
            )
            blob.download_to_filename(self.destination_file_name)
        except Exception as error:
            msg = f'Unable to download blob: "{self.blob_name}". {str(error)}'
            self.log.error(msg)
            raise GCEGetBlobException(msg) from error



# Copyright 2025 SUSE LLC
#
# This file is part of gceimgutils
#
# gceimgutils is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# gceimgutils is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with gcedeprecateimg. If not, see <http://www.gnu.org/licenses/>.

import datetime
import logging

from gceimgutils.gceutils import wait_on_operation, get_image, compute_v1
from gceimgutils.gceimgutils import GCEImageUtils
from gceimgutils.gceimgutilsExceptions import GCEDeprecateImgException

from dateutil.relativedelta import relativedelta


class GCEDeprecateImage(GCEImageUtils):
    """Class to deprecate images from GCE project"""

    # ---------------------------------------------------------------------
    def __init__(
        self,
        image_name,
        project,
        replacement_image_name=None,
        months_to_deletion=6,
        credentials_path=None,
        credentials_info=None,
        state='DEPRECATED',
        log_callback=None,
        log_level=logging.INFO
    ):
        GCEImageUtils.__init__(
            self,
            project,
            credentials_path,
            credentials_info,
            log_level,
            log_callback
        )

        self.image_name = image_name
        self.replacement_image_name = replacement_image_name
        self.months_to_deletion = months_to_deletion
        self.state = state

    # ---------------------------------------------------------------------
    def deprecate_image(self):
        """Deprecate the image"""
        kwargs = {
            'state': self.state
        }

        if self.state == 'DEPRECATED':
            delete_on = datetime.date.today() + relativedelta(
                months=int(self.months_to_deletion)
            )
            delete_timestamp = ''.join([
                delete_on.isoformat(),
                'T00:00:00.000-00:00'
            ])
            kwargs['deleted'] = delete_timestamp

        if self.state == 'DEPRECATED' and self.replacement_image_name:
            replacement = get_image(
                self.images_client,
                self.project,
                self.replacement_image_name
            )
            kwargs['replacement'] = replacement.self_link

        deprecation_status = compute_v1.DeprecationStatus(**kwargs)

        try:
            operation = self.images_client.deprecate(
                project=self.project,
                image=self.image_name,
                deprecation_status_resource=deprecation_status
            )
        except Exception as error:
            msg = (
                f'Unable to deprecate image: "{self.image_name}". '
                f'{str(error)}'
            )
            self.log.error(msg)
            raise GCEDeprecateImgException(msg) from error

        wait_on_operation(operation, self.log, 'image deprecation')



# Copyright (c) 2021 SUSE LLC
#
# This file is part of gceimgutils.
#
# gceimgutils is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# gceimgutils is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with gceimgutils.  If not, see <http://www.gnu.org/licenses/>.


class GCEImgUtilsException(Exception):
    pass


class GCEProjectCredentialsException(GCEImgUtilsException):
    pass


class GCERemoveImgException(GCEImgUtilsException):
    pass


class GCEListImgException(GCEImgUtilsException):
    pass


class GCEDeprecateImgException(GCEImgUtilsException):
    pass


class GCECreateImgException(GCEImgUtilsException):
    pass


class GCERemoveBlobException(GCEImgUtilsException):
    pass


class GCEUploadBlobException(GCEImgUtilsException):
    pass


class GCEGetBlobException(GCEImgUtilsException):
    pass



import pytest

from unittest.mock import patch, MagicMock

from gceimgutils.gceremoveblob import GCERemoveBlob


class TestGCERemoveBlob(object):
    def setup_method(self, method):
        self.storage_client = MagicMock()
        self.logger = MagicMock()

        self.kwargs = {
            'bucket_name': 'bucket',
            'blob_name': 'blob',
            'credentials_path': 'test/data/creds.json',
            'log_callback': self.logger
        }

        self.remover = GCERemoveBlob(**self.kwargs)

    @patch('gceimgutils.gceutils.storage')
    @patch('gceimgutils.gceutils.AuthorizedSession')
    def test_remove_blob(self, mock_auth_session, mock_storage):
        mock_storage.Client.return_value = self.storage_client

        self.remover.remove_blob()

    @patch('gceimgutils.gceutils.storage')
    @patch('gceimgutils.gceutils.AuthorizedSession')
    def test_deprecate_exc(self, mock_auth_session, mock_storage):
        mock_storage.Client.return_value = self.storage_client

        bucket = MagicMock()
        bucket.delete_blob.side_effect = Exception(
            'Invalid credentials!'
        )

        self.storage_client.bucket.return_value = bucket

        with pytest.raises(Exception) as error:
            self.remover.remove_blob()

        msg = 'Unable to delete blob: "blob". Invalid credentials!'
        assert str(error.value) == msg



# Copyright (c) 2021 SUSE LLC
#
# This file is part of gceimgutils
#
# gceimgutils is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# gceimgutils is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with gceimgutils.ase.  If not, see <http://www.gnu.org/licenses/>.

import datetime
import itertools
import json
import logging
import os
import random
import re
import time

from google.oauth2 import service_account
from google.api_core.extended_operation import ExtendedOperation
from google.auth.exceptions import RefreshError
from google.auth.transport.requests import AuthorizedSession
from google.cloud import compute_v1, storage

from gceimgutils.gceimgutilsExceptions import (
    GCEProjectCredentialsException,
    GCEImgUtilsException
)


# ----------------------------------------------------------------------------
def find_images_by_name(images, image_name, log_callback):
    """Return a list of images that match the given name."""
    matching_images = []
    for image in images:
        if not image.name:
            _no_name_warning(image, log_callback)
            continue
        if image_name == image.name:
            matching_images.append(image)

    return matching_images


# ----------------------------------------------------------------------------
def find_images_by_name_fragment(images, image_name_fragment, log_callback):
    """Return a list of images that match the given fragment in any part
       of the image name."""
    matching_images = []
    for image in images:
        if not image.name:
            _no_name_warning(image, log_callback)
            continue
        if image.name.find(image_name_fragment) != -1:
            matching_images.append(image)

    return matching_images


# ----------------------------------------------------------------------------
def find_images_by_name_regex_match(images, image_name_regex, log_callback):
    """Return a list of images that match the given regular expression in
       their name."""
    matching_images = []
    image_name_exp = re.compile(image_name_regex)
    for image in images:
        if not image.name:
            _no_name_warning(image, log_callback)
            continue
        if image_name_exp.match(image.name):
            matching_images.append(image)

    return matching_images


# ----------------------------------------------------------------------------
def load_credentials_from_file(credentials_file):
    with open(credentials_file) as creds_file:
        credentials_info = json.load(creds_file)

    return credentials_info


# ----------------------------------------------------------------------------
def get_credentials(
    project_name=None,
    credentials_file=None,
    credentials_info=None
):
    """Get the service account credentials for the given project"""

    if not project_name and not credentials_file and not credentials_info:
        raise GCEProjectCredentialsException(
            'Either project name, credentials file path or credentials '
            'object must be given'
        )

    if credentials_file and not os.path.exists(credentials_file):
        raise GCEProjectCredentialsException(
            'Provided credentials file "%s" not found' % credentials_file
        )

    if not credentials_file and not credentials_info:
        credentials_file = os.path.expanduser(
            '~/.config/gce/%s.json' % project_name)
        if not os.path.exists(credentials_file):
            raise GCEProjectCredentialsException(
                '"%s" credentials not found' % credentials_file
            )

    if credentials_file:
        credentials_info = load_credentials_from_file(credentials_file)

    try:
        credentials = service_account.Credentials.from_service_account_info(
            credentials_info
        )
    except Exception as error:
        raise GCEProjectCredentialsException(
            f'Could not load credentials: {error}'
        )

    try:
        # https://developers.google.com/identity/protocols/oauth2/scopes#google-sign-in
        scoped_credentials = credentials.with_scopes(['profile'])
        authed_session = AuthorizedSession(scoped_credentials)
        authed_session.get('https://www.googleapis.com/oauth2/v2/userinfo')
    except RefreshError:
        raise GCEProjectCredentialsException(
            'The provided credentials are invalid or expired: '
            '{creds_file}'.format(creds_file=credentials_file)
        )
    except Exception as error:
        raise GCEProjectCredentialsException(
            'GCP authentication failed: {error}'.format(error=error)
        )

    return credentials


# ----------------------------------------------------------------------------
def get_logger(verbose):
    """
    Return new console logger at provided log level.
    """
    if verbose:
        log_level = logging.DEBUG
    else:
        log_level = logging.INFO

    logger = logging.getLogger('gceimgutils')
    logger.setLevel(log_level)

    console_handler = logging.StreamHandler()
    console_handler.setLevel(log_level)
    console_handler.setFormatter(logging.Formatter('%(message)s'))

    logger.addHandler(console_handler)
    return logger


# ----------------------------------------------------------------------------
def get_project_images(compute_driver, project_name, deprecated=False):
    """Get the images owned by the given project"""

    current_images = []
    try:
        response = compute_driver.list(
            project=project_name
        )
    except Exception:
        return current_images

    for image in response:
        if not deprecated and image.deprecated:
            continue
        current_images.append(image)

    return current_images


# ----------------------------------------------------------------------------
def get_version():
    version_file_name = 'VERSION'
    base_path = os.path.dirname(__file__)
    version = open(os.path.join(base_path, version_file_name), 'r').read()
    return version.rstrip('\n')


# ----------------------------------------------------------------------------
def get_images_client(credentials):
    """Build the images client"""
    return compute_v1.ImagesClient(
        credentials=credentials
    )


# ----------------------------------------------------------------------------
def get_regions_client(credentials):
    """Build the regions client"""
    return compute_v1.RegionsClient(
        credentials=credentials
    )


# ----------------------------------------------------------------------------
def get_zones_client(credentials):
    """Build the zones client"""
    return compute_v1.ZonesClient(
        credentials=credentials
    )


# ----------------------------------------------------------------------------
def get_storage_client(project, credentials):
    """Build the zones client"""
    return storage.Client(
        project=project,
        credentials=credentials
    )


# ----------------------------------------------------------------------------
def _no_name_warning(image, log_callback):
    """Print a warning for images that have no name"""
    msg = 'WARNING: Found image with no name, ignoring for search results. '
    msg += 'Image ID: %s' % image['ImageId']
    log_callback.info(msg)


# ----------------------------------------------------------------------------
def image_to_dict(image):
    return {
        'kind': image.kind,
        'id': image.id,
        'creationTimestamp': image.creation_timestamp,
        'name': image.name,
        'description': image.description,
        'soureType': image.source_type,
        'rawDisk': {
            'source': image.raw_disk.source,
            'containerType': image.raw_disk.container_type
        },
        'status': image.status,
        'archiveSizeBytes': image.archive_size_bytes,
        'diskSizeGb': image.disk_size_gb,
        'licenses': [img_lic for img_lic in image.licenses],
        'family': image.family,
        'selfLink': image.self_link,
        'labelFingerprint': image.label_fingerprint,
        'guestOsFeatures': [
            {'type': feature.type_} for feature in image.guest_os_features
        ],
        'licenseCodes': [code for code in image.license_codes],
        'storageLocations': [loc for loc in image.storage_locations],
        'architecture': image.architecture
    }


# ----------------------------------------------------------------------------
def wait_on_operation(
    operation: ExtendedOperation,
    log_callback: logging.Logger,
    verbose_name: str = 'operation',
    timeout: int = 300
):
    result = operation.result(timeout=timeout)

    if operation.error_code:
        raise GCEImgUtilsException(
            f'Failed {verbose_name}: {operation.error_message}'
        )

    if operation.warnings:
        for warning in operation.warnings:
            log_callback.warning(f'{warning.code}: {warning.message}')

    return result


# ----------------------------------------------------------------------------
def get_image(images_client, project, image_name):
    """
    Retrieve GCE framework image.
    """
    return images_client.get(
        project=project,
        image=image_name
    )


# ----------------------------------------------------------------------------
def wait_on_image_ready(images_client, project, image_name):
    """
    Wait for image to be in READY state.

    If image ends up in FAILED state raise an exception.
    """
    status = None
    image = None

    while status != 'READY':
        image = get_image(images_client, project, image_name)
        status = image.status

        if status == 'FAILED':
            raise GCEImgUtilsException('Image creation failed.')

        time.sleep(5)

    return image


# ----------------------------------------------------------------------------
def str_to_bool(val: str) -> int:
    """
    This function converts a response to boolean
    """
    val = val.lower()
    if val in ('y', 'yes', 't', 'true', 'on', '1'):
        return True
    if val in ('n', 'no', 'f', 'false', 'off', '0'):
        return False
    raise ValueError(f"Invalid truth value {val}")


# ----------------------------------------------------------------------------
def get_region_list(regions_client, project):
    """
    Returns a list of regions (with random zone suffix) in status UP.
    """
    regions = regions_client.list(
        project=project
    )

    region_names = set()
    for region in regions:
        if region.status == 'UP' and region.zones:
            # we actually need a specific zone not just the region, pick one
            zone = random.choice(region.zones).split('/')[-1]
            region_names.add(zone)

    return region_names


# ----------------------------------------------------------------------------
def get_zones(zones_client, project):
    """
    Returns a list of zone names for a project.
    """
    zones = zones_client.list(project=project)
    zones = sorted(zones, key=lambda zone: zone.name)
    zones_map = {}

    for zone in zones:
        zones_map.setdefault(zone.region, []).append(zone.name)

    zones = list(zones_map.values())
    random.shuffle(zones)

    return [
        'zones/{name}'.format(name=zone) for zone in itertools.chain(
            *itertools.zip_longest(*zones)
        ) if zone is not None
    ]


# ----------------------------------------------------------------------------
def create_gce_rollout(zones_client, project):
    """
    Create a rollout policy for publishing and deprecating images.
    """
    format_str = '%Y-%m-%dT%H:%M:%SZ'
    now = datetime.datetime.now()
    zones = get_zones(zones_client, project)
    policies = {}

    for num, zone in enumerate(zones):
        rollout_time = now + datetime.timedelta(hours=num)
        policies[zone] = rollout_time.strftime(format_str)

    default = now + datetime.timedelta(hours=len(zones))

    return {
        'defaultRolloutTime': default.strftime(format_str),
        'locationRolloutPolicies': policies
    }


# ----------------------------------------------------------------------------
def blob_exists(storage_client, bucket_name, blob_name):
    """
    Return True if the blob exists in the given bucket.
    """
    bucket = storage_client.bucket(bucket_name)
    blob = bucket.blob(blob_name)
    return blob.exists()
